from app import db


class Entreprise(db.Model):
    __tablename__ = "are_entreprises"
    id = db.Column(db.Integer, primary_key=True)
    siret = db.Column(db.Text)
    nom = db.Column(db.Text)
    adresse = db.Column(db.Text)
    codepostal = db.Column(db.Text)
    ville = db.Column(db.Text)
    pays = db.Column(db.Text, default="FRANCE")
    visible = db.Column(db.Boolean, default=False)
    correspondants = db.relationship(
        "EntrepriseCorrespondant",
        backref="entreprise",
        lazy="dynamic",
        cascade="all, delete-orphan",
    )
    offres = db.relationship(
        "EntrepriseOffre",
        backref="entreprise",
        lazy="dynamic",
        cascade="all, delete-orphan",
    )

    def to_dict(self):
        return {
            "siret": self.siret,
            "nom_entreprise": self.nom,
            "adresse": self.adresse,
            "code_postal": self.codepostal,
            "ville": self.ville,
            "pays": self.pays,
        }


# class EntrepriseSite(db.Model):
#     __tablename__ = "are_sites"
#     id = db.Column(db.Integer, primary_key=True)
#     entreprise_id = db.Column(
#         db.Integer, db.ForeignKey("are_entreprises.id", ondelete="cascade")
#     )
#     nom = db.Column(db.Text)


class EntrepriseCorrespondant(db.Model):
    __tablename__ = "are_correspondants"
    id = db.Column(db.Integer, primary_key=True)
    entreprise_id = db.Column(
        db.Integer, db.ForeignKey("are_entreprises.id", ondelete="cascade")
    )
    # site_id = db.Column(db.Integer, db.ForeignKey("are_sites.id", ondelete="cascade"))
    nom = db.Column(db.Text)
    prenom = db.Column(db.Text)
    telephone = db.Column(db.Text)
    mail = db.Column(db.Text)
    poste = db.Column(db.Text)
    service = db.Column(db.Text)

    def to_dict(self):
        entreprise = Entreprise.query.filter_by(id=self.entreprise_id).first()
        return {
            "nom": self.nom,
            "prenom": self.prenom,
            "telephone": self.telephone,
            "mail": self.mail,
            "poste": self.poste,
            "service": self.service,
            "entreprise_siret": entreprise.siret,
        }


class EntrepriseContact(db.Model):
    __tablename__ = "are_contacts"
    id = db.Column(db.Integer, primary_key=True)
    date = db.Column(db.DateTime(timezone=True))
    user = db.Column(db.Integer, db.ForeignKey("user.id", ondelete="cascade"))
    entreprise = db.Column(
        db.Integer, db.ForeignKey("are_entreprises.id", ondelete="cascade")
    )
    notes = db.Column(db.Text)


class EntrepriseOffre(db.Model):
    __tablename__ = "are_offres"
    id = db.Column(db.Integer, primary_key=True)
    entreprise_id = db.Column(
        db.Integer, db.ForeignKey("are_entreprises.id", ondelete="cascade")
    )
    date_ajout = db.Column(db.DateTime(timezone=True), server_default=db.func.now())
    intitule = db.Column(db.Text)
    description = db.Column(db.Text)
    type_offre = db.Column(db.Text)
    missions = db.Column(db.Text)
    duree = db.Column(db.Text)
    expiration_date = db.Column(db.Date)
    expired = db.Column(db.Boolean, default=False)
    correspondant_id = db.Column(
        db.Integer, db.ForeignKey("are_correspondants.id", ondelete="cascade")
    )

    def to_dict(self):
        return {
            "intitule": self.intitule,
            "description": self.description,
            "type_offre": self.type_offre,
            "missions": self.missions,
            "duree": self.duree,
        }


class EntrepriseLog(db.Model):
    __tablename__ = "are_logs"
    id = db.Column(db.Integer, primary_key=True)
    date = db.Column(db.DateTime(timezone=True), server_default=db.func.now())
    authenticated_user = db.Column(db.Text)
    object = db.Column(db.Integer)
    text = db.Column(db.Text)


class EntrepriseStageApprentissage(db.Model):
    __tablename__ = "are_stages_apprentissages"
    id = db.Column(db.Integer, primary_key=True)
    entreprise_id = db.Column(
        db.Integer, db.ForeignKey("are_entreprises.id", ondelete="cascade")
    )
    etudid = db.Column(db.Integer)
    type_offre = db.Column(db.Text)
    date_debut = db.Column(db.Date)
    date_fin = db.Column(db.Date)
    formation_text = db.Column(db.Text)
    formation_scodoc = db.Column(db.Integer)
    notes = db.Column(db.Text)


class EntrepriseEnvoiOffre(db.Model):
    __tablename__ = "are_envoi_offre"
    id = db.Column(db.Integer, primary_key=True)
    sender_id = db.Column(db.Integer, db.ForeignKey("user.id", ondelete="cascade"))
    receiver_id = db.Column(db.Integer, db.ForeignKey("user.id", ondelete="cascade"))
    offre_id = db.Column(db.Integer, db.ForeignKey("are_offres.id", ondelete="cascade"))
    date_envoi = db.Column(db.DateTime(timezone=True), server_default=db.func.now())


class EntrepriseEnvoiOffreEtudiant(db.Model):
    __tablename__ = "are_envoi_offre_etudiant"
    id = db.Column(db.Integer, primary_key=True)
    sender_id = db.Column(db.Integer, db.ForeignKey("user.id", ondelete="cascade"))
    receiver_id = db.Column(
        db.Integer, db.ForeignKey("identite.id", ondelete="cascade")
    )
    offre_id = db.Column(db.Integer, db.ForeignKey("are_offres.id", ondelete="cascade"))
    date_envoi = db.Column(db.DateTime(timezone=True), server_default=db.func.now())


class EntrepriseOffreDepartement(db.Model):
    __tablename__ = "are_offre_departement"
    id = db.Column(db.Integer, primary_key=True)
    offre_id = db.Column(db.Integer, db.ForeignKey("are_offres.id", ondelete="cascade"))
    dept_id = db.Column(db.Integer, db.ForeignKey("departement.id", ondelete="cascade"))


# class EntrepriseCorrespondantDepartement(db.Model):
#     __tablename__ = "are_correspondant_departement"
#     id = db.Column(db.Integer, primary_key=True)
#     correspondant_id = db.Column(
#         db.Integer, db.ForeignKey("are_correspondants.id", ondelete="cascade")
#     )
#     dept_id = db.Column(db.Integer, db.ForeignKey("departement.id", ondelete="cascade"))


class EntreprisePreferences(db.Model):
    __tablename__ = "are_preferences"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.Text)
    value = db.Column(db.Text)

    @classmethod
    def get_email_notifications(cls):
        mail = EntreprisePreferences.query.filter_by(
            name="mail_notifications_entreprise"
        ).first()
        if mail is None:
            return ""
        else:
            return mail.value

    @classmethod
    def set_email_notifications(cls, mail: str):
        if mail != cls.get_email_notifications():
            m = EntreprisePreferences.query.filter_by(
                name="mail_notifications_entreprise"
            ).first()
            if m is None:
                prefs = EntreprisePreferences(
                    name="mail_notifications_entreprise",
                    value=mail,
                )
                db.session.add(prefs)
            else:
                m.value = mail
            db.session.commit()

    @classmethod
    def get_check_siret(cls):
        check_siret = EntreprisePreferences.query.filter_by(name="check_siret").first()
        if check_siret is None:
            return 1
        else:
            return int(check_siret.value)

    @classmethod
    def set_check_siret(cls, check_siret: int):
        cs = EntreprisePreferences.query.filter_by(name="check_siret").first()
        if cs is None:
            prefs = EntreprisePreferences(name="check_siret", value=check_siret)
            db.session.add(prefs)
        else:
            cs.value = check_siret
        db.session.commit()
